@@ -25,7 +25,7 @@
 #include <zdir.h>
 #include <zdump.h>
 #include <zerror.h>
-#include <zfence.h>
+//#include <zfence.h>
 #include <zfhs.h>
 #include <zfile.h>
 #include <zfiledlg.h>
