@@ -54,7 +54,7 @@ struct zzsdl {
 
 struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse);
 void zsdl_free(void);
-struct zzsdl *zsdl_instance();
+struct zzsdl *zsdl_instance(void);
 
 SDL_Surface *z_create_surface(struct zzsdl *zzsdl, int w, int h);
 
