@@ -21,6 +21,8 @@
 #include <windows.h>
 #endif
 
+#include <zthread.h>
+
 enum zserial_type{
     ZSERTYPE_TTY,
     ZSERTYPE_FTDI,
@@ -85,6 +87,8 @@ struct zserial{
 
 	int nolocks;
     GPtrArray *ports; // detected zserial_ports
+	MUTEX_DEFINE(close3);
+
 };
 
 struct zserial *zserial_init(void);
