@@ -51,7 +51,10 @@ gchar *zfile_fgets(GString *gs, FILE *f, int stripcomment){
 
     while (1){
         c = fgets(s, 100, f);
-        if (!c) return NULL;
+		if (!c) {
+			if (gs->len > 0) break;
+			return NULL;
+		}
         
         len = strlen(s);
         if (len>0 && s[len-1]=='\n'){ /* MAC ?*/
