@@ -135,8 +135,12 @@ int zfiledlg_open(struct zfiledlg *dlg, struct zselect *zsel, void (*fce)(void *
 
     dlg->memlist = g_ptr_array_new();
 
-	//dlg->thread = g_thread_create(zfiledlg_thread_func, dlg, TRUE, NULL);
+#ifdef Z_MACOS
+	zfiledlg_thread_func(dlg);
+#else
 	dlg->thread = g_thread_try_new("zfiledlg", zfiledlg_thread_func, dlg, NULL);
+#endif
+
     return 0;
 #else
     return -1;
