@@ -1,6 +1,6 @@
 /*
     zgptrarray.c - GPtrArray extension
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2020  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -11,6 +11,7 @@
 #include <zgptrarray.h>
 
 #include <stdlib.h>
+#include <string.h>
 
 void zg_ptr_array_free_all(GPtrArray *array){
     int i;
@@ -42,3 +43,18 @@ void zg_ptr_array_qsort (GPtrArray *array,
     
     qsort( (void *)array->pdata, array->len, sizeof(gpointer), compar);
 }
+
+// case sensitive
+// 0 = found, 1 = not found
+int zg_ptr_array_find_str(GPtrArray *array, char *needle){
+	int i;
+	char *item;
+
+	if (!array) return 1;
+	for (i = 0; (long)i < (long)array->len; i++){
+		item = (char*)g_ptr_array_index(array, i);
+		if (strcmp(item, needle) == 0) return 0;
+	}
+	return 1;
+}
+
