@@ -150,7 +150,7 @@ gpointer zhdkeyb_main(gpointer xxx){
     zhdkeyb_clear(zhdkeyb);
 #if 0 && defined(Z_HAVE_SDL) && defined(Z_HAVE_LIBPNG)
     SDL_Surface *png = z_png_create(icon_tucnak23, sizeof(icon_tucnak23));
-    if (png) zinternal("Can't create icon_tucnak23, currupted executable?");
+    if (png) zinternal("Can't create icon_tucnak23, corrupted executable?");
 
     zhdkeyb_setdir(zhdkeyb); 	
     zhdkeyb_cursor(zhdkeyb, 0);
