@@ -1,6 +1,6 @@
 /*
     zhttp.c - http client
-    Copyright (C) 2012-2020 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2012-2022 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -85,8 +85,8 @@ void zhttp_free(struct zhttp *http){
     g_free(http->serveraddr);
     g_free(http->page);
 	g_free(http->url);
-	g_hash_table_remove_all(http->cookies);
-	g_hash_table_remove_all(http->headers);
+	g_hash_table_destroy(http->cookies);
+	g_hash_table_destroy(http->headers);
 
 	zhttp_post_free(http);
 	g_free(http->datastr);
