@@ -94,7 +94,7 @@ struct zhttpd *zhttpd_init(struct zselect *zsel, int port, int loopback){
 	g_ptr_array_set_free_func(zhttpd->conns, (GDestroyNotify)zhttpd_free_conn);
     zhttpd->bindings = g_ptr_array_new();
 	g_ptr_array_set_free_func(zhttpd->bindings, (GDestroyNotify)zhttpd_free_binding);
-    dbg("zhttpd active\n");
+	dbg("zhttpd active on TCP port %d\n", ntohs(sin.sin_port));
 
     return zhttpd;
 x:;
@@ -104,6 +104,12 @@ x:;
     return NULL;
 }
 
+void zhttpd_handlers(struct zhttpd *httpd, void(*conn_new)(struct zhttpconn *conn), void(*conn_close)(struct zhttpconn *conn)){
+	httpd->conn_new = conn_new;
+	httpd->conn_close = conn_close;
+}
+
+
 void zhttpd_free(struct zhttpd *zhttpd){
     if (!zhttpd) return;
 
@@ -119,14 +125,14 @@ void zhttpd_free(struct zhttpd *zhttpd){
 }
 
 void zhttpd_free_conn(struct zhttpconn *conn){
-    if (conn->sock >= 0) {
-        zselect_set(conn->zhttpd->zsel, conn->sock, NULL, NULL, NULL, NULL);
-        closesocket(conn->sock);
-    }
-	zselect_timer_kill(conn->zhttpd->zsel, conn->ping_timer_id);
+	zhttpd_close_conn(conn);
+	if (conn->ws_ping_timer_id > 0) zselect_timer_kill(conn->zhttpd->zsel, conn->ws_ping_timer_id);
 
     g_string_free(conn->request, TRUE);
+	MUTEX_LOCK(conn->response);
     if (conn->response) zbinbuf_free(conn->response);
+	MUTEX_UNLOCK(conn->response);
+	MUTEX_FREE(conn->response);
 
 	g_free(conn->req_path);
 	g_free(conn->req_args);
@@ -135,6 +141,16 @@ void zhttpd_free_conn(struct zhttpconn *conn){
     g_free(conn);
 }
 
+void zhttpd_close_conn(struct zhttpconn *conn){
+	if (conn->sock >= 0) {
+		zselect_set(conn->zhttpd->zsel, conn->sock, NULL, NULL, NULL, NULL);
+		closesocket(conn->sock);
+		conn->sock = -1;
+        if (conn->zhttpd->conn_close) conn->zhttpd->conn_close(conn);
+	}
+}
+
+
 void zhttpd_free_binding(struct zhttpdbinding *b){
 
 	g_regex_unref(b->regex);
@@ -153,6 +169,7 @@ void zhttpd_accept_handler(void *arg){
 	conn->zhttpd = zhttpd;
 	conn->memlist = g_ptr_array_new_with_free_func(g_free);
 	conn->response_headers = g_ptr_array_new_with_free_func((GDestroyNotify)zhttpd_free_header);
+	MUTEX_INIT(conn->response);
 
     socklen = sizeof(conn->peer);
     sock = accept(zhttpd->sock, (struct sockaddr *)&conn->peer, &socklen);
@@ -169,7 +186,7 @@ void zhttpd_accept_handler(void *arg){
     
     zselect_set(zhttpd->zsel, conn->sock, zhttpd_read_handler, NULL, NULL, conn);
     g_ptr_array_add(zhttpd->conns, conn);
-	conn->ping_timer_id = zselect_timer_new(zhttpd->zsel, 30000, zhttpd_ws_ping_timer, conn);
+    if (zhttpd->conn_new) zhttpd->conn_new(conn);
 }
 
 void zhttpd_read_handler(void *arg){
@@ -214,7 +231,10 @@ void zhttpd_read_handler(void *arg){
         zhttpd_post(conn);
     }else{
         zhttpd_response(conn, 400, NULL);
-        zbinbuf_sprintfa(conn->response, "<html><body>Bad request</body></html>");
+		MUTEX_LOCK(conn->response); // v dobe cteni asi nemusi byt
+		zbinbuf_sprintfa(conn->response, "<html><body>Bad request</body></html>");
+		MUTEX_UNLOCK(conn->response);
+
     }
 /*    for (i = 0; i < 1000; i++){
         g_string_append_c(conn->response, '0' + (i / 100));
@@ -228,38 +248,62 @@ void zhttpd_read_handler(void *arg){
 void zhttpd_write_handler(void *arg){
     struct zhttpconn *conn;
     int ret, len;
+	int remove_after = 0;
 
     conn = (struct zhttpconn*)arg;
 
     
-    len = conn->response->len - conn->response_i;
+	MUTEX_LOCK(conn->response);
+	len = conn->response->len - conn->response_i;
+    //printf("len=%d\n", len);
     if (len > 1400) len = 1400;
 	if (len > 0)
 	{
 		ret = send(conn->sock, conn->response->buf + conn->response_i, len, 0);
-		if (ret <= 0){
-			g_ptr_array_remove(conn->zhttpd->conns, conn);
-			return;
+		int err = z_sock_errno;
+        //printf("send(%d) = %d\n", len, ret);
+		if (ret <= 0){	
+			if (err != EWOULDBLOCK){
+				if (conn->is_stream){
+					zhttpd_close_conn(conn);
+				}
+				else{
+					g_ptr_array_remove(conn->zhttpd->conns, conn);
+				}
+			}
+		}else{
+			if (conn->is_stream){
+				zbinbuf_erase(conn->response, 0, ret); 
+			}else{
+				conn->response_i += ret;
+			}
 		}
-		conn->response_i += ret;
 	}else{
 		void *read_handler = zselect_get(conn->zhttpd->zsel, conn->sock, H_READ);
 		if (read_handler == NULL) {
-			g_ptr_array_remove(conn->zhttpd->conns, conn);
+			if (conn->is_stream){
+				zselect_set_write(conn->zhttpd->zsel, conn->sock, NULL, conn);
+			}else{
+				remove_after = 1;
+			}
 		}else{
 			zselect_set_write(conn->zhttpd->zsel, conn->sock, NULL, conn);
 		}
-        return;
     }
+
+	MUTEX_UNLOCK(conn->response);
+	if (remove_after) g_ptr_array_remove(conn->zhttpd->conns, conn);
+
+
 }
 
-int zhttpd_flush(struct zhttpconn *conn){
+/*int zhttpd_flush(struct zhttpconn *conn){
     int ret, len;
     len = conn->response->len - conn->response_i;
 	ret = send(conn->sock, conn->response->buf + conn->response_i, len, 0);
 	if (ret > 0) conn->response_i += ret;
 	return ret;
-}
+} */
 
 void zhttpd_write_response_header(struct zhttpconn *conn){
 	int i;
@@ -283,7 +327,9 @@ void zhttpd_write_response_header(struct zhttpconn *conn){
 		g_string_append_printf(sb, "%s: %s\r\n", h->key, h->value);
 	}
 	g_string_append(sb, "\r\n");
+	MUTEX_LOCK(conn->response);
 	zbinbuf_prepend(conn->response, sb->str, sb->len);
+	MUTEX_UNLOCK(conn->response);
 	g_string_free(sb, TRUE);
 }
 
@@ -365,7 +411,9 @@ void zhttpd_get(struct zhttpconn *conn){
 
     
 	zhttpd_response(conn, 404, "text/plain");
+	MUTEX_LOCK(conn->response);
 	zbinbuf_sprintfa(conn->response, "Not found");
+	MUTEX_UNLOCK(conn->response);
 }
 
 void zhttpd_post(struct zhttpconn *conn){
@@ -405,7 +453,9 @@ void zhttpd_post(struct zhttpconn *conn){
 
     
 	zhttpd_response(conn, 404, "text/plain");
+	MUTEX_LOCK(conn->response);
 	zbinbuf_sprintfa(conn->response, "Not found");
+	MUTEX_UNLOCK(conn->response);
 }
 
 struct zhttpdbinding *zhttpd_add_binding(struct zhttpd *zhttpd, char *regex,
@@ -499,11 +549,16 @@ void zhttpd_file_handler(struct zhttpconn *conn){
 	if (file == NULL)
 	{
 		zhttpd_response(conn, 404, "text/plain");
+		MUTEX_LOCK(conn->response);
 		zbinbuf_sprintfa(conn->response, "Not found");
-	}else{
+		MUTEX_UNLOCK(conn->response);
+	}
+	else{
 		contenttype = g_strdup(zhttpd_get_mime(conn->req_path));
 		zhttpd_response(conn, 200, contenttype);
+		MUTEX_LOCK(conn->response);
 		zbinbuf_append(conn->response, file);
+		MUTEX_UNLOCK(conn->response);
 		g_free(contenttype);
 		g_free(file);
 	}
@@ -532,6 +587,8 @@ void zhttpd_ws_handshake_handler(struct zhttpconn *conn){
 	if (protocol != NULL) zhttpd_add_header(conn, "Sec-WebSocket-Protocol", protocol);
 	zselect_set_read(conn->zhttpd->zsel, conn->sock, zhttpd_ws_read_handler, conn);
 	conn->is_ws = 1;
+	conn->ws_ping_timer_id = zselect_timer_new(conn->zhttpd->zsel, 30000, zhttpd_ws_ping_timer, conn);
+
 }
 
 void zhttpd_ws_read_handler(void *arg)
@@ -616,6 +673,7 @@ void zhttpd_ws_send(struct zhttpconn *conn, int opcode, char *s, int len){
 
 	if (len < 0) len = strlen(s);
 
+	MUTEX_LOCK(conn->response);
 	b = opcode | 0x80; // FIN
 	zbinbuf_append_bin(conn->response, &b, 1);
 	if (len <= 125){
@@ -631,6 +689,8 @@ void zhttpd_ws_send(struct zhttpconn *conn, int opcode, char *s, int len){
 	}	
 
 	zbinbuf_append_bin(conn->response, s, len);
+	MUTEX_UNLOCK(conn->response);
+
 	zselect_set_write(conn->zhttpd->zsel, conn->sock, zhttpd_write_handler, conn);
 }
 
@@ -649,9 +709,28 @@ void zhttpd_ws_send_all(struct zhttpd *zhttpd, int opcode, char *s, int len){
 void zhttpd_ws_ping_timer(void *arg){
 	struct zhttpconn *conn = (struct zhttpconn*)arg;
 
+	MUTEX_LOCK(conn->response);
 	zbinbuf_append_bin(conn->response, "\x89\0", 2);
+	MUTEX_UNLOCK(conn->response);
 
-	conn->ping_timer_id = zselect_timer_new(conn->zhttpd->zsel, 30000, zhttpd_ws_ping_timer, conn);
+	conn->ws_ping_timer_id = zselect_timer_new(conn->zhttpd->zsel, 30000, zhttpd_ws_ping_timer, conn);
 	zselect_set_write(conn->zhttpd->zsel, conn->sock, zhttpd_write_handler, conn);
 
 }
+
+
+
+void zhttpd_write(struct zhttpconn *conn, const void *data, int len){
+    if (conn->sock == -1) return;
+	MUTEX_LOCK(conn->response);
+	zbinbuf_append_bin(conn->response, data, len);
+	MUTEX_UNLOCK(conn->response);
+	zselect_set_write(conn->zhttpd->zsel, conn->sock, zhttpd_write_handler, conn);
+}
+
+int http_response_buf_len(struct zhttpconn *conn){
+	MUTEX_LOCK(conn->response);
+	int len = conn->response->len;
+	MUTEX_UNLOCK(conn->response);
+	return len;
+}
