@@ -1,6 +1,6 @@
 /*
     select.c - main application loop
-    Copyright (C) 2002-2020  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2022  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
     This program is free software; you can redistribute it and/or
@@ -13,7 +13,6 @@
 #include <eprintf.h>
 #include <zerror.h>
 #include <zselect.h>
-#include <zfence.h>
 #include <zsock.h>
 #include <zdebug.h>
 #include <zlist.h>
@@ -267,7 +266,6 @@ static void zselect_timers_check(struct zselect *zsel)
 			if (zsel->profile) gettimeofday(&start, NULL);
 			//dbg("%s\n", tt->fname);
             tt->func(tt->arg);
-			//zfence_check(tt->fname);
 			if (zsel->profile) zselect_handle_profile(zsel, &start, tt->func, tt->fname);
             g_free(tt);
             CHK_BH;
@@ -803,7 +801,6 @@ void zselect_loop(struct zselect *zsel)
 					if (zsel->profile) gettimeofday(&start, NULL);
 					//dbg("%s\n", zfd->read_fname);
                     zfd->read_func(zfd->arg);
-					//zfence_check(zfd->read_fname);
 					if (zsel->profile) zselect_handle_profile(zsel, &start, zfd->read_func, zfd->read_fname);
                     CHK_BH;
                 }
@@ -814,7 +811,6 @@ void zselect_loop(struct zselect *zsel)
 					if (zsel->profile) gettimeofday(&start, NULL);
 					//dbg("%s\n", zfd->write_fname);
 					zfd->write_func(zfd->arg);
-					//zfence_check(zfd->write_fname);
 					if (zsel->profile) zselect_handle_profile(zsel, &start, zfd->write_func, zfd->write_fname);
                     CHK_BH;
                 }
@@ -825,7 +821,6 @@ void zselect_loop(struct zselect *zsel)
 					if (zsel->profile) gettimeofday(&start, NULL);
 					//dbg("%s\n", zfd->error_fname);
 					zfd->error_func(zfd->arg);
-					//zfence_check(zfd->error_fname);
 					if (zsel->profile) zselect_handle_profile(zsel, &start, zfd->error_func, zfd->error_fname);
                     CHK_BH;
                 }
