@@ -15,6 +15,7 @@
 #include <zdebug.h>
 #include <zerror.h>
 #include <zgptrarray.h>
+#include <zthread.h>
 #include <zmisc.h>
 #include <zmsgbox.h>
 
@@ -126,9 +127,11 @@ static int zserial_win32_write(struct zserial *zser, void *data, int len){
 }
 
 static int zserial_win32_close(struct zserial *zser){
+	MUTEX_LOCK(zser->close3);
 	if (zser->handle != INVALID_HANDLE_VALUE) CloseHandle(zser->handle);
 	zser->handle = INVALID_HANDLE_VALUE;
-    return 0;
+	MUTEX_UNLOCK(zser->close3);
+	return 0;
 }
 
 static int zserial_win32_dtr(struct zserial *zser, int on){
